// Copyright (c) FIRST and other WPILib contributors.
// Open Source Software; you can modify and/or share it under the terms of
// the WPILib BSD license file in the root directory of this project.

#ifndef NTCORE_NTCORE_H_
#define NTCORE_NTCORE_H_

/* C API */
#include "ntcore_c.h"

#ifdef __cplusplus
/* C++ API */
#include "ntcore_cpp.h"
#endif /* __cplusplus */

#endif  // NTCORE_NTCORE_H_
