
 

// This file is autogenerated. DO NOT EDIT

#pragma once
#include <robotpy_build.h>


#include <../_impl/include/frc/smartdashboard/Field2d.h>

#include <networktables/NTSendableBuilder.h>



#include <rpygen/nt__NTSendable.hpp>

namespace rpygen {

using namespace frc;


template <typename CfgBase>
using PyTrampolineCfgBase_frc__Field2d =
    PyTrampolineCfg_nt__NTSendable<
CfgBase
>;

template <typename CfgBase = EmptyTrampolineCfg>
struct PyTrampolineCfg_frc__Field2d :
    PyTrampolineCfgBase_frc__Field2d< CfgBase>
{
    using Base = frc::Field2d;

    using override_base_InitSendable_RTNTSendableBuilder = frc::Field2d;
};


template <typename PyTrampolineBase, typename PyTrampolineCfg>
using PyTrampolineBase_frc__Field2d =
    PyTrampoline_nt__NTSendable<
        PyTrampolineBase
        
        , PyTrampolineCfg
    >
;

template <typename PyTrampolineBase, typename PyTrampolineCfg>
struct PyTrampoline_frc__Field2d : PyTrampolineBase_frc__Field2d<PyTrampolineBase, PyTrampolineCfg> {
    using PyTrampolineBase_frc__Field2d<PyTrampolineBase, PyTrampolineCfg>::PyTrampolineBase_frc__Field2d;



#ifndef RPYGEN_DISABLE_InitSendable_RTNTSendableBuilder
    void InitSendable(nt::NTSendableBuilder& builder) override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_InitSendable_RTNTSendableBuilder;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(void), LookupBase,
            "initSendable", builder);
        return CxxCallBase::InitSendable(builder);
    }
#endif




};

}; // namespace rpygen
