
 

// This file is autogenerated. DO NOT EDIT

#pragma once
#include <robotpy_build.h>


#include <../_api/include/ctre/phoenix/tasking/IProcessable.h>





namespace rpygen {

using namespace ctre::phoenix::tasking;



template <typename CfgBase = EmptyTrampolineCfg>
struct PyTrampolineCfg_ctre__phoenix__tasking__IProcessable :
    CfgBase
{
    using Base = ctre::phoenix::tasking::IProcessable;

    using override_base_Process_v = ctre::phoenix::tasking::IProcessable;
};


template <typename PyTrampolineBase, typename PyTrampolineCfg>
struct PyTrampoline_ctre__phoenix__tasking__IProcessable : PyTrampolineBase, virtual py::trampoline_self_life_support {
    using PyTrampolineBase::PyTrampolineBase;



#ifndef RPYGEN_DISABLE_Process_v
    void Process() override {
        using LookupBase = typename PyTrampolineCfg::Base;
        RPYBUILD_OVERRIDE_PURE_NAME(IProcessable, PYBIND11_TYPE(void), LookupBase,
            "process", Process, );
    }
#endif




};

}; // namespace rpygen
