
 

// This file is autogenerated. DO NOT EDIT

#pragma once
#include <robotpy_build.h>


#include <../_impl/include/frc/smartdashboard/SendableChooserBase.h>

#include <wpi/sendable/SendableBuilder.h>



#include <rpygen/nt__NTSendable.hpp>

namespace rpygen {

using namespace frc;


template <typename CfgBase>
using PyTrampolineCfgBase_frc__SendableChooserBase =
    PyTrampolineCfg_nt__NTSendable<
CfgBase
>;

template <typename CfgBase = EmptyTrampolineCfg>
struct PyTrampolineCfg_frc__SendableChooserBase :
    PyTrampolineCfgBase_frc__SendableChooserBase< CfgBase>
{
    using Base = frc::SendableChooserBase;

};


template <typename PyTrampolineBase, typename PyTrampolineCfg>
using PyTrampolineBase_frc__SendableChooserBase =
    PyTrampoline_nt__NTSendable<
        PyTrampolineBase
        
        , PyTrampolineCfg
    >
;

template <typename PyTrampolineBase, typename PyTrampolineCfg>
struct PyTrampoline_frc__SendableChooserBase : PyTrampolineBase_frc__SendableChooserBase<PyTrampolineBase, PyTrampolineCfg> {
    using PyTrampolineBase_frc__SendableChooserBase<PyTrampolineBase, PyTrampolineCfg>::PyTrampolineBase_frc__SendableChooserBase;





    using frc::SendableChooserBase::kDefault;
    using frc::SendableChooserBase::kOptions;
    using frc::SendableChooserBase::kSelected;
    using frc::SendableChooserBase::kActive;
    using frc::SendableChooserBase::kInstance;
    using frc::SendableChooserBase::m_defaultChoice;
    using frc::SendableChooserBase::m_selected;
    using frc::SendableChooserBase::m_haveSelected;
    using frc::SendableChooserBase::m_activeEntries;
    using frc::SendableChooserBase::m_instance;

};

}; // namespace rpygen
