
 

// This file is autogenerated. DO NOT EDIT

#pragma once
#include <robotpy_build.h>


#include <../src/TimedCommandRobot.h>




#include <rpygen/frc__TimedRobot.hpp>

namespace rpygen {

using namespace frc2;


template <typename CfgBase>
using PyTrampolineCfgBase_frc2__TimedCommandRobot =
    PyTrampolineCfg_frc__TimedRobot<
CfgBase
>;

template <typename CfgBase = EmptyTrampolineCfg>
struct PyTrampolineCfg_frc2__TimedCommandRobot :
    PyTrampolineCfgBase_frc2__TimedCommandRobot< CfgBase>
{
    using Base = frc2::TimedCommandRobot;

    using override_base_RobotPeriodic_v = frc2::TimedCommandRobot;
};


template <typename PyTrampolineBase, typename PyTrampolineCfg>
using PyTrampolineBase_frc2__TimedCommandRobot =
    PyTrampoline_frc__TimedRobot<
        PyTrampolineBase
        
        , PyTrampolineCfg
    >
;

template <typename PyTrampolineBase, typename PyTrampolineCfg>
struct PyTrampoline_frc2__TimedCommandRobot : PyTrampolineBase_frc2__TimedCommandRobot<PyTrampolineBase, PyTrampolineCfg> {
    using PyTrampolineBase_frc2__TimedCommandRobot<PyTrampolineBase, PyTrampolineCfg>::PyTrampolineBase_frc2__TimedCommandRobot;



#ifndef RPYGEN_DISABLE_RobotPeriodic_v
    void RobotPeriodic() override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_RobotPeriodic_v;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(void), LookupBase,
            "robotPeriodic", );
        return CxxCallBase::RobotPeriodic();
    }
#endif




};

}; // namespace rpygen
