
 

// This file is autogenerated. DO NOT EDIT

#pragma once
#include <robotpy_build.h>


#include <../../_impl/include/frc/interfaces/Accelerometer.h>





namespace rpygen {

using namespace frc;



template <typename CfgBase = EmptyTrampolineCfg>
struct PyTrampolineCfg_frc__Accelerometer :
    CfgBase
{
    using Base = frc::Accelerometer;

    using override_base_SetRange_TRange = frc::Accelerometer;
    using override_base_GetX_v = frc::Accelerometer;
    using override_base_GetY_v = frc::Accelerometer;
    using override_base_GetZ_v = frc::Accelerometer;
};


template <typename PyTrampolineBase, typename PyTrampolineCfg>
struct PyTrampoline_frc__Accelerometer : PyTrampolineBase, virtual py::trampoline_self_life_support {
    using PyTrampolineBase::PyTrampolineBase;



#ifndef RPYGEN_DISABLE_SetRange_TRange
    void SetRange(frc::Accelerometer::Range range) override {
        using LookupBase = typename PyTrampolineCfg::Base;
        RPYBUILD_OVERRIDE_PURE_NAME(Accelerometer, PYBIND11_TYPE(void), LookupBase,
            "setRange", SetRange, range);
    }
#endif

#ifndef RPYGEN_DISABLE_GetX_v
    double GetX() override {
        using LookupBase = typename PyTrampolineCfg::Base;
        RPYBUILD_OVERRIDE_PURE_NAME(Accelerometer, PYBIND11_TYPE(double), LookupBase,
            "getX", GetX, );
    }
#endif

#ifndef RPYGEN_DISABLE_GetY_v
    double GetY() override {
        using LookupBase = typename PyTrampolineCfg::Base;
        RPYBUILD_OVERRIDE_PURE_NAME(Accelerometer, PYBIND11_TYPE(double), LookupBase,
            "getY", GetY, );
    }
#endif

#ifndef RPYGEN_DISABLE_GetZ_v
    double GetZ() override {
        using LookupBase = typename PyTrampolineCfg::Base;
        RPYBUILD_OVERRIDE_PURE_NAME(Accelerometer, PYBIND11_TYPE(double), LookupBase,
            "getZ", GetZ, );
    }
#endif




};

}; // namespace rpygen
