
 

// This file is autogenerated. DO NOT EDIT

#pragma once
#include <robotpy_build.h>


#include <../../build/temp.linux-armv7l-3.10/dlstatic/revlib/include/rev/AnalogInput.h>





namespace rpygen {

using namespace rev;



template <typename CfgBase = EmptyTrampolineCfg>
struct PyTrampolineCfg_rev__AnalogInput :
    CfgBase
{
    using Base = rev::AnalogInput;

    using override_base_KGetVoltage_v = rev::AnalogInput;
    using override_base_KGetPosition_v = rev::AnalogInput;
    using override_base_SetPositionConversionFactor_d = rev::AnalogInput;
    using override_base_KGetPositionConversionFactor_v = rev::AnalogInput;
};


template <typename PyTrampolineBase, typename PyTrampolineCfg>
struct PyTrampoline_rev__AnalogInput : PyTrampolineBase, virtual py::trampoline_self_life_support {
    using PyTrampolineBase::PyTrampolineBase;



#ifndef RPYGEN_DISABLE_KGetVoltage_v
    double GetVoltage() const override {
        using LookupBase = typename PyTrampolineCfg::Base;
        RPYBUILD_OVERRIDE_PURE_NAME(AnalogInput, PYBIND11_TYPE(double), LookupBase,
            "getVoltage", GetVoltage, );
    }
#endif

#ifndef RPYGEN_DISABLE_KGetPosition_v
    double GetPosition() const override {
        using LookupBase = typename PyTrampolineCfg::Base;
        RPYBUILD_OVERRIDE_PURE_NAME(AnalogInput, PYBIND11_TYPE(double), LookupBase,
            "getPosition", GetPosition, );
    }
#endif

#ifndef RPYGEN_DISABLE_SetPositionConversionFactor_d
    REVLibError SetPositionConversionFactor(double factor) override {
        using LookupBase = typename PyTrampolineCfg::Base;
        RPYBUILD_OVERRIDE_PURE_NAME(AnalogInput, PYBIND11_TYPE(REVLibError), LookupBase,
            "setPositionConversionFactor", SetPositionConversionFactor, factor);
    }
#endif

#ifndef RPYGEN_DISABLE_KGetPositionConversionFactor_v
    double GetPositionConversionFactor() const override {
        using LookupBase = typename PyTrampolineCfg::Base;
        RPYBUILD_OVERRIDE_PURE_NAME(AnalogInput, PYBIND11_TYPE(double), LookupBase,
            "getPositionConversionFactor", GetPositionConversionFactor, );
    }
#endif




};

}; // namespace rpygen
