
 

// This file is autogenerated. DO NOT EDIT

#pragma once
#include <robotpy_build.h>


#include <../../_impl/include/frc/spline/QuinticHermiteSpline.h>




#include <rpygen/frc__Spline.hpp>

namespace rpygen {

using namespace frc;


template <typename CfgBase>
using PyTrampolineCfgBase_frc__QuinticHermiteSpline =
    PyTrampolineCfg_frc__Spline<5, 
CfgBase
>;

template <typename CfgBase = EmptyTrampolineCfg>
struct PyTrampolineCfg_frc__QuinticHermiteSpline :
    PyTrampolineCfgBase_frc__QuinticHermiteSpline< CfgBase>
{
    using Base = frc::QuinticHermiteSpline;

};


template <typename PyTrampolineBase, typename PyTrampolineCfg>
using PyTrampolineBase_frc__QuinticHermiteSpline =
    PyTrampoline_frc__Spline<
        PyTrampolineBase
        , 5
        , PyTrampolineCfg
    >
;

template <typename PyTrampolineBase, typename PyTrampolineCfg>
struct PyTrampoline_frc__QuinticHermiteSpline : PyTrampolineBase_frc__QuinticHermiteSpline<PyTrampolineBase, PyTrampolineCfg> {
    using PyTrampolineBase_frc__QuinticHermiteSpline<PyTrampolineBase, PyTrampolineCfg>::PyTrampolineBase_frc__QuinticHermiteSpline;






};

}; // namespace rpygen
