#pragma once

namespace ctre {
namespace phoenix {
namespace motion {

enum SetValueMotionProfile {
	Disable = 0, Enable = 1, Hold = 2,
};

} // namespace motion
} // namespace phoenix
} // namespace ctre
