/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class com_ctre_phoenix_motion_BuffTrajPointStreamJNI */

#ifndef _Included_com_ctre_phoenix_motion_BuffTrajPointStreamJNI
#define _Included_com_ctre_phoenix_motion_BuffTrajPointStreamJNI
#ifdef __cplusplus
extern "C" {
#endif

/*
 * Class:     com_ctre_phoenix_motion_BuffTrajPointStreamJNI
 * Method:    Create
 * Signature: (V)J
 */
JNIEXPORT jlong JNICALL Java_com_ctre_phoenix_motion_BuffTrajPointStreamJNI_Create1
  (JNIEnv *, jclass);

/*
 * Class:     com_ctre_phoenix_motion_BuffTrajPointStreamJNI
 * Method:    DestroyAll
 * Signature: (V)V
 */
JNIEXPORT void JNICALL Java_com_ctre_phoenix_motion_BuffTrajPointStreamJNI_DestroyAll
  (JNIEnv *, jclass);


/*
 * Class:     com_ctre_phoenix_motion_BuffTrajPointStreamJNI
 * Method:    Destroy
 * Signature: (J)V
 */
JNIEXPORT int JNICALL Java_com_ctre_phoenix_motion_BuffTrajPointStreamJNI_Destroy
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_motion_BuffTrajPointStreamJNI
 * Method:    Clear
 * Signature: (J)V
 */
JNIEXPORT int JNICALL Java_com_ctre_phoenix_motion_BuffTrajPointStreamJNI_Clear
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_motion_BuffTrajPointStreamJNI
 * Method:    Write
 * Signature: (JDDDDDDIIZZIZ)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_motion_BuffTrajPointStreamJNI_Write
  (JNIEnv *, jclass, jlong, jdouble, jdouble, jdouble, jdouble, jdouble, jdouble, jint, jint, jboolean, jboolean, jint, jboolean);

#ifdef __cplusplus
}
#endif
#endif