
 

// This file is autogenerated. DO NOT EDIT

#pragma once
#include <robotpy_build.h>


#include <../../build/temp.linux-armv7l-3.10/dlstatic/revlib/include/rev/SparkMaxAlternateEncoder.h>



#define RPYGEN_DISABLE_KGetSparkMaxFeedbackDeviceID_v

#include <rpygen/rev__RelativeEncoder.hpp>

namespace rpygen {

using namespace rev;


template <typename CfgBase>
using PyTrampolineCfgBase_rev__SparkMaxAlternateEncoder =
    PyTrampolineCfg_rev__RelativeEncoder<
CfgBase
>;

template <typename CfgBase = EmptyTrampolineCfg>
struct PyTrampolineCfg_rev__SparkMaxAlternateEncoder :
    PyTrampolineCfgBase_rev__SparkMaxAlternateEncoder< CfgBase>
{
    using Base = rev::SparkMaxAlternateEncoder;

    using override_base_KGetPosition_v = rev::SparkMaxAlternateEncoder;
    using override_base_KGetVelocity_v = rev::SparkMaxAlternateEncoder;
    using override_base_SetPosition_d = rev::SparkMaxAlternateEncoder;
    using override_base_SetPositionConversionFactor_d = rev::SparkMaxAlternateEncoder;
    using override_base_SetVelocityConversionFactor_d = rev::SparkMaxAlternateEncoder;
    using override_base_KGetPositionConversionFactor_v = rev::SparkMaxAlternateEncoder;
    using override_base_KGetVelocityConversionFactor_v = rev::SparkMaxAlternateEncoder;
    using override_base_SetAverageDepth_j = rev::SparkMaxAlternateEncoder;
    using override_base_SetMeasurementPeriod_j = rev::SparkMaxAlternateEncoder;
    using override_base_KGetAverageDepth_v = rev::SparkMaxAlternateEncoder;
    using override_base_KGetMeasurementPeriod_v = rev::SparkMaxAlternateEncoder;
    using override_base_KGetCountsPerRevolution_v = rev::SparkMaxAlternateEncoder;
    using override_base_SetInverted_b = rev::SparkMaxAlternateEncoder;
    using override_base_KGetInverted_v = rev::SparkMaxAlternateEncoder;
};


template <typename PyTrampolineBase, typename PyTrampolineCfg>
using PyTrampolineBase_rev__SparkMaxAlternateEncoder =
    PyTrampoline_rev__RelativeEncoder<
        PyTrampolineBase
        
        , PyTrampolineCfg
    >
;

template <typename PyTrampolineBase, typename PyTrampolineCfg>
struct PyTrampoline_rev__SparkMaxAlternateEncoder : PyTrampolineBase_rev__SparkMaxAlternateEncoder<PyTrampolineBase, PyTrampolineCfg> {
    using PyTrampolineBase_rev__SparkMaxAlternateEncoder<PyTrampolineBase, PyTrampolineCfg>::PyTrampolineBase_rev__SparkMaxAlternateEncoder;



#ifndef RPYGEN_DISABLE_KGetPosition_v
    double GetPosition() const override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_KGetPosition_v;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(double), LookupBase,
            "getPosition", );
        return CxxCallBase::GetPosition();
    }
#endif

#ifndef RPYGEN_DISABLE_KGetVelocity_v
    double GetVelocity() const override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_KGetVelocity_v;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(double), LookupBase,
            "getVelocity", );
        return CxxCallBase::GetVelocity();
    }
#endif

#ifndef RPYGEN_DISABLE_SetPosition_d
    REVLibError SetPosition(double position) override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_SetPosition_d;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(REVLibError), LookupBase,
            "setPosition", position);
        return CxxCallBase::SetPosition(position);
    }
#endif

#ifndef RPYGEN_DISABLE_SetPositionConversionFactor_d
    REVLibError SetPositionConversionFactor(double factor) override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_SetPositionConversionFactor_d;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(REVLibError), LookupBase,
            "setPositionConversionFactor", factor);
        return CxxCallBase::SetPositionConversionFactor(factor);
    }
#endif

#ifndef RPYGEN_DISABLE_SetVelocityConversionFactor_d
    REVLibError SetVelocityConversionFactor(double factor) override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_SetVelocityConversionFactor_d;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(REVLibError), LookupBase,
            "setVelocityConversionFactor", factor);
        return CxxCallBase::SetVelocityConversionFactor(factor);
    }
#endif

#ifndef RPYGEN_DISABLE_KGetPositionConversionFactor_v
    double GetPositionConversionFactor() const override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_KGetPositionConversionFactor_v;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(double), LookupBase,
            "getPositionConversionFactor", );
        return CxxCallBase::GetPositionConversionFactor();
    }
#endif

#ifndef RPYGEN_DISABLE_KGetVelocityConversionFactor_v
    double GetVelocityConversionFactor() const override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_KGetVelocityConversionFactor_v;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(double), LookupBase,
            "getVelocityConversionFactor", );
        return CxxCallBase::GetVelocityConversionFactor();
    }
#endif

#ifndef RPYGEN_DISABLE_SetAverageDepth_j
    REVLibError SetAverageDepth(uint32_t depth) override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_SetAverageDepth_j;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(REVLibError), LookupBase,
            "setAverageDepth", depth);
        return CxxCallBase::SetAverageDepth(depth);
    }
#endif

#ifndef RPYGEN_DISABLE_SetMeasurementPeriod_j
    REVLibError SetMeasurementPeriod(uint32_t period_ms) override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_SetMeasurementPeriod_j;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(REVLibError), LookupBase,
            "setMeasurementPeriod", period_ms);
        return CxxCallBase::SetMeasurementPeriod(period_ms);
    }
#endif

#ifndef RPYGEN_DISABLE_KGetAverageDepth_v
    uint32_t GetAverageDepth() const override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_KGetAverageDepth_v;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(uint32_t), LookupBase,
            "getAverageDepth", );
        return CxxCallBase::GetAverageDepth();
    }
#endif

#ifndef RPYGEN_DISABLE_KGetMeasurementPeriod_v
    uint32_t GetMeasurementPeriod() const override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_KGetMeasurementPeriod_v;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(uint32_t), LookupBase,
            "getMeasurementPeriod", );
        return CxxCallBase::GetMeasurementPeriod();
    }
#endif

#ifndef RPYGEN_DISABLE_KGetCountsPerRevolution_v
    uint32_t GetCountsPerRevolution() const override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_KGetCountsPerRevolution_v;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(uint32_t), LookupBase,
            "getCountsPerRevolution", );
        return CxxCallBase::GetCountsPerRevolution();
    }
#endif

#ifndef RPYGEN_DISABLE_SetInverted_b
    REVLibError SetInverted(bool inverted) override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_SetInverted_b;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(REVLibError), LookupBase,
            "setInverted", inverted);
        return CxxCallBase::SetInverted(inverted);
    }
#endif

#ifndef RPYGEN_DISABLE_KGetInverted_v
    bool GetInverted() const override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_KGetInverted_v;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(bool), LookupBase,
            "getInverted", );
        return CxxCallBase::GetInverted();
    }
#endif




};

}; // namespace rpygen
