
 

// This file is autogenerated. DO NOT EDIT

#pragma once
#include <robotpy_build.h>


#include <../_impl/include/frc/MotorSafety.h>

#include <wpi/SmallString.h>




namespace rpygen {

using namespace frc;



template <typename CfgBase = EmptyTrampolineCfg>
struct PyTrampolineCfg_frc__MotorSafety :
    CfgBase
{
    using Base = frc::MotorSafety;

    using override_base_StopMotor_v = frc::MotorSafety;
    using override_base_KGetDescription_v = frc::MotorSafety;
};


template <typename PyTrampolineBase, typename PyTrampolineCfg>
struct PyTrampoline_frc__MotorSafety : PyTrampolineBase, virtual py::trampoline_self_life_support {
    using PyTrampolineBase::PyTrampolineBase;



#ifndef RPYGEN_DISABLE_StopMotor_v
    void StopMotor() override {
        using LookupBase = typename PyTrampolineCfg::Base;
        RPYBUILD_OVERRIDE_PURE_NAME(MotorSafety, PYBIND11_TYPE(void), LookupBase,
            "stopMotor", StopMotor, );
    }
#endif

#ifndef RPYGEN_DISABLE_KGetDescription_v
    std::string GetDescription() const override {
        using LookupBase = typename PyTrampolineCfg::Base;
        RPYBUILD_OVERRIDE_PURE_NAME(MotorSafety, PYBIND11_TYPE(std::string), LookupBase,
            "getDescription", GetDescription, );
    }
#endif




};

}; // namespace rpygen
