
 

// This file is autogenerated. DO NOT EDIT

#pragma once
#include <robotpy_build.h>


#include <../../_impl/include/frc/drive/RobotDriveBase.h>

#include <frc/SpeedController.h>
#include <wpi/SmallString.h>



#include <rpygen/frc__MotorSafety.hpp>

namespace rpygen {

using namespace frc;


template <typename CfgBase>
using PyTrampolineCfgBase_frc__RobotDriveBase =
    PyTrampolineCfg_frc__MotorSafety<
CfgBase
>;

template <typename CfgBase = EmptyTrampolineCfg>
struct PyTrampolineCfg_frc__RobotDriveBase :
    PyTrampolineCfgBase_frc__RobotDriveBase< CfgBase>
{
    using Base = frc::RobotDriveBase;

    using override_base_StopMotor_v = frc::RobotDriveBase;
    using override_base_KGetDescription_v = frc::RobotDriveBase;
};


template <typename PyTrampolineBase, typename PyTrampolineCfg>
using PyTrampolineBase_frc__RobotDriveBase =
    PyTrampoline_frc__MotorSafety<
        PyTrampolineBase
        
        , PyTrampolineCfg
    >
;

template <typename PyTrampolineBase, typename PyTrampolineCfg>
struct PyTrampoline_frc__RobotDriveBase : PyTrampolineBase_frc__RobotDriveBase<PyTrampolineBase, PyTrampolineCfg> {
    using PyTrampolineBase_frc__RobotDriveBase<PyTrampolineBase, PyTrampolineCfg>::PyTrampolineBase_frc__RobotDriveBase;



#ifndef RPYGEN_DISABLE_StopMotor_v
    void StopMotor() override {
        using LookupBase = typename PyTrampolineCfg::Base;
        RPYBUILD_OVERRIDE_PURE_NAME(RobotDriveBase, PYBIND11_TYPE(void), LookupBase,
            "stopMotor", StopMotor, );
    }
#endif

#ifndef RPYGEN_DISABLE_KGetDescription_v
    std::string GetDescription() const override {
        using LookupBase = typename PyTrampolineCfg::Base;
        RPYBUILD_OVERRIDE_PURE_NAME(RobotDriveBase, PYBIND11_TYPE(std::string), LookupBase,
            "getDescription", GetDescription, );
    }
#endif


#ifndef RPYBLD_DISABLE_ApplyDeadband_d_d
  #ifndef RPYBLD_UDISABLE_frc__RobotDriveBase_ApplyDeadband
    using frc::RobotDriveBase::ApplyDeadband;
    #define RPYBLD_UDISABLE_frc__RobotDriveBase_ApplyDeadband
  #endif
#endif
#ifndef RPYBLD_DISABLE_Desaturate_Tspan_double_
  #ifndef RPYBLD_UDISABLE_frc__RobotDriveBase_Desaturate
    using frc::RobotDriveBase::Desaturate;
    #define RPYBLD_UDISABLE_frc__RobotDriveBase_Desaturate
  #endif
#endif

    using frc::RobotDriveBase::m_deadband;
    using frc::RobotDriveBase::m_maxOutput;

};

}; // namespace rpygen
