
 

// This file is autogenerated. DO NOT EDIT

#pragma once
#include <robotpy_build.h>


#include <../_impl/include/frc/PS4Controller.h>




#include <rpygen/frc__GenericHID.hpp>

namespace rpygen {

using namespace frc;


template <typename CfgBase>
using PyTrampolineCfgBase_frc__PS4Controller =
    PyTrampolineCfg_frc__GenericHID<
CfgBase
>;

template <typename CfgBase = EmptyTrampolineCfg>
struct PyTrampolineCfg_frc__PS4Controller :
    PyTrampolineCfgBase_frc__PS4Controller< CfgBase>
{
    using Base = frc::PS4Controller;

};


template <typename PyTrampolineBase, typename PyTrampolineCfg>
using PyTrampolineBase_frc__PS4Controller =
    PyTrampoline_frc__GenericHID<
        PyTrampolineBase
        
        , PyTrampolineCfg
    >
;

template <typename PyTrampolineBase, typename PyTrampolineCfg>
struct PyTrampoline_frc__PS4Controller : PyTrampolineBase_frc__PS4Controller<PyTrampolineBase, PyTrampolineCfg> {
    using PyTrampolineBase_frc__PS4Controller<PyTrampolineBase, PyTrampolineCfg>::PyTrampolineBase_frc__PS4Controller;






};

}; // namespace rpygen
