
 

// This file is autogenerated. DO NOT EDIT

#pragma once
#include <robotpy_build.h>


#include <../src/include/frc2/command/Subsystem.h>

#include <frc2/command/Command.h>




namespace rpygen {

using namespace frc2;



template <typename CfgBase = EmptyTrampolineCfg>
struct PyTrampolineCfg_frc2__Subsystem :
    CfgBase
{
    using Base = frc2::Subsystem;

    using override_base_Periodic_v = frc2::Subsystem;
    using override_base_SimulationPeriodic_v = frc2::Subsystem;
};


template <typename PyTrampolineBase, typename PyTrampolineCfg>
struct PyTrampoline_frc2__Subsystem : PyTrampolineBase, virtual py::trampoline_self_life_support {
    using PyTrampolineBase::PyTrampolineBase;



#ifndef RPYGEN_DISABLE_Periodic_v
    void Periodic() override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_Periodic_v;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(void), LookupBase,
            "periodic", );
        return CxxCallBase::Periodic();
    }
#endif

#ifndef RPYGEN_DISABLE_SimulationPeriodic_v
    void SimulationPeriodic() override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_SimulationPeriodic_v;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(void), LookupBase,
            "simulationPeriodic", );
        return CxxCallBase::SimulationPeriodic();
    }
#endif




};

}; // namespace rpygen
