/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class com_ctre_phoenix_OrchestraJNI */

#ifndef _Included_com_ctre_phoenix_OrchestraJNI
#define _Included_com_ctre_phoenix_OrchestraJNI
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     com_ctre_phoenix_OrchestraJNI
 * Method:    JNI_new_Orchestra
 * Signature: ()J
 */
JNIEXPORT jlong JNICALL Java_com_ctre_phoenix_music_OrchestraJNI_JNI_1new_1Orchestra
  (JNIEnv *, jclass);

/*
 * Class:     com_ctre_phoenix_OrchestraJNI
 * Method:    JNI_destroy_Orchestra
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_music_OrchestraJNI_JNI_1destroy_1Orchestra
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_OrchestraJNI
 * Method:    JNI_LoadMusic
 * Signature: (JLjava/lang/String;)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_music_OrchestraJNI_JNI_1LoadMusic
  (JNIEnv *, jclass, jlong, jstring);

/*
 * Class:     com_ctre_phoenix_OrchestraJNI
 * Method:    JNI_Play
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_music_OrchestraJNI_JNI_1Play
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_OrchestraJNI
 * Method:    JNI_Stop
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_music_OrchestraJNI_JNI_1Stop
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_OrchestraJNI
 * Method:    JNI_Pause
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_music_OrchestraJNI_JNI_1Pause
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_OrchestraJNI
 * Method:    JNI_IsPlaying
 * Signature: (J)Z
 */
JNIEXPORT jboolean JNICALL Java_com_ctre_phoenix_music_OrchestraJNI_JNI_1IsPlaying
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_OrchestraJNI
 * Method:    JNI_GetCurrentTime
 * Signature: (J)Z
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_music_OrchestraJNI_JNI_1GetCurrentTime
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_OrchestraJNI
 * Method:    JNI_ClearInstruments
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_music_OrchestraJNI_JNI_1ClearInstruments
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_OrchestraJNI
 * Method:    JNI_AddInstrument
 * Signature: (JJ)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_music_OrchestraJNI_JNI_1AddInstrument
  (JNIEnv *, jclass, jlong, jlong);

/*
 * Class:     com_ctre_phoenix_OrchestraJNI
 * Method:    JNI_GetDescription
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_music_OrchestraJNI_JNI_1GetDescription
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_OrchestraJNI
 * Method:    JNI_SetDescription
 * Signature: (JLjava/lang/String;)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_music_OrchestraJNI_JNI_1SetDescription
  (JNIEnv *, jclass, jlong, jstring);

#ifdef __cplusplus
}
#endif
#endif
