
 

// This file is autogenerated. DO NOT EDIT

#pragma once
#include <robotpy_build.h>


#include <../_cci/include/ctre/phoenix/core/GadgeteerUartClient.h>





namespace rpygen {




template <typename CfgBase = EmptyTrampolineCfg>
struct PyTrampolineCfg___IGadgeteerUartClient :
    CfgBase
{
    using Base = ::IGadgeteerUartClient;

    using override_base_GetGadgeteerStatus_RTGadgeteerUartStatus = ::IGadgeteerUartClient;
};


template <typename PyTrampolineBase, typename PyTrampolineCfg>
struct PyTrampoline___IGadgeteerUartClient : PyTrampolineBase, virtual py::trampoline_self_life_support {
    using PyTrampolineBase::PyTrampolineBase;



#ifndef RPYGEN_DISABLE_GetGadgeteerStatus_RTGadgeteerUartStatus
    int GetGadgeteerStatus(IGadgeteerUartClient::GadgeteerUartStatus& status) override {
        using LookupBase = typename PyTrampolineCfg::Base;
        RPYBUILD_OVERRIDE_PURE_NAME(IGadgeteerUartClient, PYBIND11_TYPE(int), LookupBase,
            "getGadgeteerStatus", GetGadgeteerStatus, status);
    }
#endif




};

}; // namespace rpygen
