// Copyright (c) FIRST and other WPILib contributors.
// Open Source Software; you can modify and/or share it under the terms of
// the WPILib BSD license file in the root directory of this project.

#pragma once

namespace frc {
enum class CompressorConfigType {
  Disabled = 0,
  Digital = 1,
  Analog = 2,
  Hybrid = 3
};

}  // namespace frc
