
 

// This file is autogenerated. DO NOT EDIT

#pragma once
#include <robotpy_build.h>


#include <../../build/temp.linux-armv7l-3.10/dlstatic/revlib/include/rev/CANSparkMaxLowLevel.h>




#include <rpygen/frc__MotorController.hpp>

namespace rpygen {

using namespace rev;


template <typename CfgBase>
using PyTrampolineCfgBase_rev__CANSparkMaxLowLevel =
    PyTrampolineCfg_frc__MotorController<
CfgBase
>;

template <typename CfgBase = EmptyTrampolineCfg>
struct PyTrampolineCfg_rev__CANSparkMaxLowLevel :
    PyTrampolineCfgBase_rev__CANSparkMaxLowLevel< CfgBase>
{
    using Base = rev::CANSparkMaxLowLevel;

};


template <typename PyTrampolineBase, typename PyTrampolineCfg>
using PyTrampolineBase_rev__CANSparkMaxLowLevel =
    PyTrampoline_frc__MotorController<
        PyTrampolineBase
        
        , PyTrampolineCfg
    >
;

template <typename PyTrampolineBase, typename PyTrampolineCfg>
struct PyTrampoline_rev__CANSparkMaxLowLevel : PyTrampolineBase_rev__CANSparkMaxLowLevel<PyTrampolineBase, PyTrampolineCfg> {
    using PyTrampolineBase_rev__CANSparkMaxLowLevel<PyTrampolineBase, PyTrampolineCfg>::PyTrampolineBase_rev__CANSparkMaxLowLevel;




#ifndef RPYBLD_DISABLE_GetPeriodicStatus0_v
  #ifndef RPYBLD_UDISABLE_rev__CANSparkMaxLowLevel_GetPeriodicStatus0
    using rev::CANSparkMaxLowLevel::GetPeriodicStatus0;
    #define RPYBLD_UDISABLE_rev__CANSparkMaxLowLevel_GetPeriodicStatus0
  #endif
#endif
#ifndef RPYBLD_DISABLE_GetPeriodicStatus1_v
  #ifndef RPYBLD_UDISABLE_rev__CANSparkMaxLowLevel_GetPeriodicStatus1
    using rev::CANSparkMaxLowLevel::GetPeriodicStatus1;
    #define RPYBLD_UDISABLE_rev__CANSparkMaxLowLevel_GetPeriodicStatus1
  #endif
#endif
#ifndef RPYBLD_DISABLE_GetPeriodicStatus2_v
  #ifndef RPYBLD_UDISABLE_rev__CANSparkMaxLowLevel_GetPeriodicStatus2
    using rev::CANSparkMaxLowLevel::GetPeriodicStatus2;
    #define RPYBLD_UDISABLE_rev__CANSparkMaxLowLevel_GetPeriodicStatus2
  #endif
#endif
#ifndef RPYBLD_DISABLE_GetSafeFloat_f
  #ifndef RPYBLD_UDISABLE_rev__CANSparkMaxLowLevel_GetSafeFloat
    using rev::CANSparkMaxLowLevel::GetSafeFloat;
    #define RPYBLD_UDISABLE_rev__CANSparkMaxLowLevel_GetSafeFloat
  #endif
#endif


};

}; // namespace rpygen
