
 

// This file is autogenerated. DO NOT EDIT

#pragma once
#include <robotpy_build.h>


#include <../../_impl/include/frc/CounterBase.h>





namespace rpygen {

using namespace frc;



template <typename CfgBase = EmptyTrampolineCfg>
struct PyTrampolineCfg_frc__CounterBase :
    CfgBase
{
    using Base = frc::CounterBase;

    using override_base_KGet_v = frc::CounterBase;
    using override_base_Reset_v = frc::CounterBase;
    using override_base_KGetPeriod_v = frc::CounterBase;
    using override_base_SetMaxPeriod_Tsecond_t = frc::CounterBase;
    using override_base_KGetStopped_v = frc::CounterBase;
    using override_base_KGetDirection_v = frc::CounterBase;
};


template <typename PyTrampolineBase, typename PyTrampolineCfg>
struct PyTrampoline_frc__CounterBase : PyTrampolineBase, virtual py::trampoline_self_life_support {
    using PyTrampolineBase::PyTrampolineBase;



#ifndef RPYGEN_DISABLE_KGet_v
    int Get() const override {
        using LookupBase = typename PyTrampolineCfg::Base;
        RPYBUILD_OVERRIDE_PURE_NAME(CounterBase, PYBIND11_TYPE(int), LookupBase,
            "get", Get, );
    }
#endif

#ifndef RPYGEN_DISABLE_Reset_v
    void Reset() override {
        using LookupBase = typename PyTrampolineCfg::Base;
        RPYBUILD_OVERRIDE_PURE_NAME(CounterBase, PYBIND11_TYPE(void), LookupBase,
            "reset", Reset, );
    }
#endif

#ifndef RPYGEN_DISABLE_KGetPeriod_v
    units::second_t GetPeriod() const override {
        using LookupBase = typename PyTrampolineCfg::Base;
        RPYBUILD_OVERRIDE_PURE_NAME(CounterBase, PYBIND11_TYPE(units::second_t), LookupBase,
            "getPeriod", GetPeriod, );
    }
#endif

#ifndef RPYGEN_DISABLE_SetMaxPeriod_Tsecond_t
    void SetMaxPeriod(units::second_t maxPeriod) override {
        using LookupBase = typename PyTrampolineCfg::Base;
        RPYBUILD_OVERRIDE_PURE_NAME(CounterBase, PYBIND11_TYPE(void), LookupBase,
            "setMaxPeriod", SetMaxPeriod, maxPeriod);
    }
#endif

#ifndef RPYGEN_DISABLE_KGetStopped_v
    bool GetStopped() const override {
        using LookupBase = typename PyTrampolineCfg::Base;
        RPYBUILD_OVERRIDE_PURE_NAME(CounterBase, PYBIND11_TYPE(bool), LookupBase,
            "getStopped", GetStopped, );
    }
#endif

#ifndef RPYGEN_DISABLE_KGetDirection_v
    bool GetDirection() const override {
        using LookupBase = typename PyTrampolineCfg::Base;
        RPYBUILD_OVERRIDE_PURE_NAME(CounterBase, PYBIND11_TYPE(bool), LookupBase,
            "getDirection", GetDirection, );
    }
#endif




};

}; // namespace rpygen
