# This file is automatically generated, DO NOT EDIT
# fmt: off

from os.path import abspath, join, dirname, exists
_root = abspath(dirname(__file__))

# runtime dependencies
import wpilib._init_wpilib
import wpilib.interfaces._init_interfaces
import wpilib._impl._init_wpilibc
import wpimath._impl._init_wpimath_cpp
import wpimath._controls._init_controls
import wpimath.geometry._init_geometry
import wpimath.filter._init_filter
import wpimath.kinematics._init_kinematics
import wpimath.spline._init_spline
import hal._init_wpiHal
import wpiutil._init_wpiutil
import _pyntcore._init_ntcore
