#pragma once

namespace ctre {
namespace phoenix {
namespace sensors {

/** Enumerated type for status frame types. */
enum PigeonIMU_ControlFrame {
	PigeonIMU_CondStatus_Control_1 = 0x00042800,
};

} // namespace sensors
} // namespace phoenix
} // namespace ctre
