# fmt: off
# This file is automatically generated, DO NOT EDIT

from os.path import abspath, join, dirname
_root = abspath(dirname(__file__))

libinit_import = "wpiutil._init_wpiutil"
depends = []
pypi_package = 'robotpy-wpiutil'

def get_include_dirs():
    return [join(_root, "include"), join(_root, "rpy-include"), join(_root, "src", "type_casters")]

def get_library_dirs():
    return [join(_root, "lib")]

def get_library_dirs_rel():
    return ['lib']

def get_library_names():
    return ['wpiutil']

def get_library_full_names():
    return ['libwpiutil.so']

def get_type_casters_cfg(casters):
    casters.update({'wpi::array': {'hdr': 'wpi_array_type_caster.h'}, 'wpi::span': {'hdr': 'wpi_span_type_caster.h'}, 'wpi::SmallSet': {'hdr': 'wpi_smallset_type_caster.h'}, 'wpi::SmallVector': {'hdr': 'wpi_smallvector_type_caster.h'}, 'wpi::SmallVectorImpl': {'hdr': 'wpi_smallvectorimpl_type_caster.h'}})

def get_type_casters(casters):
    t = {}
    get_type_casters_cfg(t)
    for k, v in t.items():
        if "hdr" in v:
            casters[k] = v["hdr"]