
 

// This file is autogenerated. DO NOT EDIT

#pragma once
#include <robotpy_build.h>


#include <../_api/include/ctre/phoenix/led/BaseStandardAnimation.h>




#include <rpygen/ctre__phoenix__led__Animation.hpp>

namespace rpygen {

using namespace ctre::phoenix::led;


template <typename CfgBase>
using PyTrampolineCfgBase_ctre__phoenix__led__BaseStandardAnimation =
    PyTrampolineCfg_ctre__phoenix__led__Animation<
CfgBase
>;

template <typename CfgBase = EmptyTrampolineCfg>
struct PyTrampolineCfg_ctre__phoenix__led__BaseStandardAnimation :
    PyTrampolineCfgBase_ctre__phoenix__led__BaseStandardAnimation< CfgBase>
{
    using Base = ctre::phoenix::led::BaseStandardAnimation;

};


template <typename PyTrampolineBase, typename PyTrampolineCfg>
using PyTrampolineBase_ctre__phoenix__led__BaseStandardAnimation =
    PyTrampoline_ctre__phoenix__led__Animation<
        PyTrampolineBase
        
        , PyTrampolineCfg
    >
;

template <typename PyTrampolineBase, typename PyTrampolineCfg>
struct PyTrampoline_ctre__phoenix__led__BaseStandardAnimation : PyTrampolineBase_ctre__phoenix__led__BaseStandardAnimation<PyTrampolineBase, PyTrampolineCfg> {
    using PyTrampolineBase_ctre__phoenix__led__BaseStandardAnimation<PyTrampolineBase, PyTrampolineCfg>::PyTrampolineBase_ctre__phoenix__led__BaseStandardAnimation;






};

}; // namespace rpygen
