
 

// This file is autogenerated. DO NOT EDIT

#pragma once
#include <robotpy_build.h>


#include <../_impl/include/frc/PneumaticHub.h>

#include <wpi/sendable/SendableBuilder.h>
#include <frc/Compressor.h>
#include <frc/Solenoid.h>
#include <frc/DoubleSolenoid.h>



#include <rpygen/frc__PneumaticsBase.hpp>

namespace rpygen {

using namespace frc;


template <typename CfgBase>
using PyTrampolineCfgBase_frc__PneumaticHub =
    PyTrampolineCfg_frc__PneumaticsBase<
CfgBase
>;

template <typename CfgBase = EmptyTrampolineCfg>
struct PyTrampolineCfg_frc__PneumaticHub :
    PyTrampolineCfgBase_frc__PneumaticHub< CfgBase>
{
    using Base = frc::PneumaticHub;

    using override_base_KGetCompressor_v = frc::PneumaticHub;
    using override_base_DisableCompressor_v = frc::PneumaticHub;
    using override_base_EnableCompressorDigital_v = frc::PneumaticHub;
    using override_base_EnableCompressorAnalog_Tpounds_per_square_inch_t_Tpounds_per_square_inch_t = frc::PneumaticHub;
    using override_base_EnableCompressorHybrid_Tpounds_per_square_inch_t_Tpounds_per_square_inch_t = frc::PneumaticHub;
    using override_base_KGetCompressorConfigType_v = frc::PneumaticHub;
    using override_base_KGetPressureSwitch_v = frc::PneumaticHub;
    using override_base_KGetCompressorCurrent_v = frc::PneumaticHub;
    using override_base_SetSolenoids_i_i = frc::PneumaticHub;
    using override_base_KGetSolenoids_v = frc::PneumaticHub;
    using override_base_KGetModuleNumber_v = frc::PneumaticHub;
    using override_base_KGetSolenoidDisabledList_v = frc::PneumaticHub;
    using override_base_FireOneShot_i = frc::PneumaticHub;
    using override_base_SetOneShotDuration_i_Tsecond_t = frc::PneumaticHub;
    using override_base_KCheckSolenoidChannel_i = frc::PneumaticHub;
    using override_base_CheckAndReserveSolenoids_i = frc::PneumaticHub;
    using override_base_UnreserveSolenoids_i = frc::PneumaticHub;
    using override_base_ReserveCompressor_v = frc::PneumaticHub;
    using override_base_UnreserveCompressor_v = frc::PneumaticHub;
    using override_base_MakeSolenoid_i = frc::PneumaticHub;
    using override_base_MakeDoubleSolenoid_i_i = frc::PneumaticHub;
    using override_base_MakeCompressor_v = frc::PneumaticHub;
    using override_base_KGetAnalogVoltage_i = frc::PneumaticHub;
    using override_base_KGetPressure_i = frc::PneumaticHub;
};


template <typename PyTrampolineBase, typename PyTrampolineCfg>
using PyTrampolineBase_frc__PneumaticHub =
    PyTrampoline_frc__PneumaticsBase<
        PyTrampolineBase
        
        , PyTrampolineCfg
    >
;

template <typename PyTrampolineBase, typename PyTrampolineCfg>
struct PyTrampoline_frc__PneumaticHub : PyTrampolineBase_frc__PneumaticHub<PyTrampolineBase, PyTrampolineCfg> {
    using PyTrampolineBase_frc__PneumaticHub<PyTrampolineBase, PyTrampolineCfg>::PyTrampolineBase_frc__PneumaticHub;



#ifndef RPYGEN_DISABLE_KGetCompressor_v
    bool GetCompressor() const override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_KGetCompressor_v;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(bool), LookupBase,
            "getCompressor", );
        return CxxCallBase::GetCompressor();
    }
#endif

#ifndef RPYGEN_DISABLE_DisableCompressor_v
    void DisableCompressor() override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_DisableCompressor_v;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(void), LookupBase,
            "disableCompressor", );
        return CxxCallBase::DisableCompressor();
    }
#endif

#ifndef RPYGEN_DISABLE_EnableCompressorDigital_v
    void EnableCompressorDigital() override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_EnableCompressorDigital_v;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(void), LookupBase,
            "enableCompressorDigital", );
        return CxxCallBase::EnableCompressorDigital();
    }
#endif

#ifndef RPYGEN_DISABLE_EnableCompressorAnalog_Tpounds_per_square_inch_t_Tpounds_per_square_inch_t
    void EnableCompressorAnalog(units::pounds_per_square_inch_t minPressure, units::pounds_per_square_inch_t maxPressure) override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_EnableCompressorAnalog_Tpounds_per_square_inch_t_Tpounds_per_square_inch_t;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(void), LookupBase,
            "enableCompressorAnalog", minPressure, maxPressure);
        return CxxCallBase::EnableCompressorAnalog(minPressure, maxPressure);
    }
#endif

#ifndef RPYGEN_DISABLE_EnableCompressorHybrid_Tpounds_per_square_inch_t_Tpounds_per_square_inch_t
    void EnableCompressorHybrid(units::pounds_per_square_inch_t minPressure, units::pounds_per_square_inch_t maxPressure) override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_EnableCompressorHybrid_Tpounds_per_square_inch_t_Tpounds_per_square_inch_t;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(void), LookupBase,
            "enableCompressorHybrid", minPressure, maxPressure);
        return CxxCallBase::EnableCompressorHybrid(minPressure, maxPressure);
    }
#endif

#ifndef RPYGEN_DISABLE_KGetCompressorConfigType_v
    CompressorConfigType GetCompressorConfigType() const override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_KGetCompressorConfigType_v;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(CompressorConfigType), LookupBase,
            "getCompressorConfigType", );
        return CxxCallBase::GetCompressorConfigType();
    }
#endif

#ifndef RPYGEN_DISABLE_KGetPressureSwitch_v
    bool GetPressureSwitch() const override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_KGetPressureSwitch_v;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(bool), LookupBase,
            "getPressureSwitch", );
        return CxxCallBase::GetPressureSwitch();
    }
#endif

#ifndef RPYGEN_DISABLE_KGetCompressorCurrent_v
    units::ampere_t GetCompressorCurrent() const override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_KGetCompressorCurrent_v;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(units::ampere_t), LookupBase,
            "getCompressorCurrent", );
        return CxxCallBase::GetCompressorCurrent();
    }
#endif

#ifndef RPYGEN_DISABLE_SetSolenoids_i_i
    void SetSolenoids(int mask, int values) override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_SetSolenoids_i_i;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(void), LookupBase,
            "setSolenoids", mask, values);
        return CxxCallBase::SetSolenoids(mask, values);
    }
#endif

#ifndef RPYGEN_DISABLE_KGetSolenoids_v
    int GetSolenoids() const override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_KGetSolenoids_v;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(int), LookupBase,
            "getSolenoids", );
        return CxxCallBase::GetSolenoids();
    }
#endif

#ifndef RPYGEN_DISABLE_KGetModuleNumber_v
    int GetModuleNumber() const override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_KGetModuleNumber_v;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(int), LookupBase,
            "getModuleNumber", );
        return CxxCallBase::GetModuleNumber();
    }
#endif

#ifndef RPYGEN_DISABLE_KGetSolenoidDisabledList_v
    int GetSolenoidDisabledList() const override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_KGetSolenoidDisabledList_v;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(int), LookupBase,
            "getSolenoidDisabledList", );
        return CxxCallBase::GetSolenoidDisabledList();
    }
#endif

#ifndef RPYGEN_DISABLE_FireOneShot_i
    void FireOneShot(int index) override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_FireOneShot_i;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(void), LookupBase,
            "fireOneShot", index);
        return CxxCallBase::FireOneShot(index);
    }
#endif

#ifndef RPYGEN_DISABLE_SetOneShotDuration_i_Tsecond_t
    void SetOneShotDuration(int index, units::second_t duration) override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_SetOneShotDuration_i_Tsecond_t;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(void), LookupBase,
            "setOneShotDuration", index, duration);
        return CxxCallBase::SetOneShotDuration(index, duration);
    }
#endif

#ifndef RPYGEN_DISABLE_KCheckSolenoidChannel_i
    bool CheckSolenoidChannel(int channel) const override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_KCheckSolenoidChannel_i;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(bool), LookupBase,
            "checkSolenoidChannel", channel);
        return CxxCallBase::CheckSolenoidChannel(channel);
    }
#endif

#ifndef RPYGEN_DISABLE_CheckAndReserveSolenoids_i
    int CheckAndReserveSolenoids(int mask) override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_CheckAndReserveSolenoids_i;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(int), LookupBase,
            "checkAndReserveSolenoids", mask);
        return CxxCallBase::CheckAndReserveSolenoids(mask);
    }
#endif

#ifndef RPYGEN_DISABLE_UnreserveSolenoids_i
    void UnreserveSolenoids(int mask) override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_UnreserveSolenoids_i;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(void), LookupBase,
            "unreserveSolenoids", mask);
        return CxxCallBase::UnreserveSolenoids(mask);
    }
#endif

#ifndef RPYGEN_DISABLE_ReserveCompressor_v
    bool ReserveCompressor() override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_ReserveCompressor_v;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(bool), LookupBase,
            "reserveCompressor", );
        return CxxCallBase::ReserveCompressor();
    }
#endif

#ifndef RPYGEN_DISABLE_UnreserveCompressor_v
    void UnreserveCompressor() override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_UnreserveCompressor_v;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(void), LookupBase,
            "unreserveCompressor", );
        return CxxCallBase::UnreserveCompressor();
    }
#endif

#ifndef RPYGEN_DISABLE_MakeSolenoid_i
    Solenoid MakeSolenoid(int channel) override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_MakeSolenoid_i;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(Solenoid), LookupBase,
            "makeSolenoid", channel);
        return CxxCallBase::MakeSolenoid(channel);
    }
#endif

#ifndef RPYGEN_DISABLE_MakeDoubleSolenoid_i_i
    DoubleSolenoid MakeDoubleSolenoid(int forwardChannel, int reverseChannel) override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_MakeDoubleSolenoid_i_i;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(DoubleSolenoid), LookupBase,
            "makeDoubleSolenoid", forwardChannel, reverseChannel);
        return CxxCallBase::MakeDoubleSolenoid(forwardChannel, reverseChannel);
    }
#endif

#ifndef RPYGEN_DISABLE_MakeCompressor_v
    Compressor MakeCompressor() override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_MakeCompressor_v;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(Compressor), LookupBase,
            "makeCompressor", );
        return CxxCallBase::MakeCompressor();
    }
#endif

#ifndef RPYGEN_DISABLE_KGetAnalogVoltage_i
    units::volt_t GetAnalogVoltage(int channel) const override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_KGetAnalogVoltage_i;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(units::volt_t), LookupBase,
            "getAnalogVoltage", channel);
        return CxxCallBase::GetAnalogVoltage(channel);
    }
#endif

#ifndef RPYGEN_DISABLE_KGetPressure_i
    units::pounds_per_square_inch_t GetPressure(int channel) const override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_KGetPressure_i;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(units::pounds_per_square_inch_t), LookupBase,
            "getPressure", channel);
        return CxxCallBase::GetPressure(channel);
    }
#endif




};

}; // namespace rpygen
