
 

// This file is autogenerated. DO NOT EDIT

#pragma once
#include <robotpy_build.h>


#include <../_api/include/ctre/phoenix/signals/IOutputSignal.h>





namespace rpygen {

using namespace ctre::phoenix::signals;



template <typename CfgBase = EmptyTrampolineCfg>
struct PyTrampolineCfg_ctre__phoenix__signals__IOutputSignal :
    CfgBase
{
    using Base = ctre::phoenix::signals::IOutputSignal;

    using override_base_Set_d = ctre::phoenix::signals::IOutputSignal;
};


template <typename PyTrampolineBase, typename PyTrampolineCfg>
struct PyTrampoline_ctre__phoenix__signals__IOutputSignal : PyTrampolineBase, virtual py::trampoline_self_life_support {
    using PyTrampolineBase::PyTrampolineBase;



#ifndef RPYGEN_DISABLE_Set_d
    void Set(double value) override {
        using LookupBase = typename PyTrampolineCfg::Base;
        RPYBUILD_OVERRIDE_PURE_NAME(IOutputSignal, PYBIND11_TYPE(void), LookupBase,
            "set", Set, value);
    }
#endif




};

}; // namespace rpygen
