/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class com_ctre_phoenix_platform_can_PlatformCANJNI */

#ifndef _Included_com_ctre_phoenix_platform_can_PlatformCANJNI
#define _Included_com_ctre_phoenix_platform_can_PlatformCANJNI
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     com_ctre_phoenix_platform_can_PlatformCANJNI
 * Method:    JNI_RegisterCANbus
 * Signature: ([C)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_platform_can_PlatformCANJNI_JNI_1RegisterCANbus
  (JNIEnv *, jclass, jcharArray);

/*
 * Class:     com_ctre_phoenix_platform_can_PlatformCANJNI
 * Method:    JNI_DestroyAll
 * Signature: ()I
 */
//JNIEXPORT jint JNICALL Java_com_ctre_phoenix_platform_can_PlatformCANJNI_JNI_1DestroyAll
//  (JNIEnv *, jclass);

/*
 * Class:     com_ctre_phoenix_platform_can_PlatformCANJNI
 * Method:    JNI_StartAll
 * Signature: ()I
 */
//JNIEXPORT jint JNICALL Java_com_ctre_phoenix_platform_can_PlatformCANJNI_JNI_1StartAll
//  (JNIEnv *, jclass);

/*
 * Class:     com_ctre_phoenix_platform_can_PlatformCANJNI
 * Method:    JNI_SetAutocacheLevel
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_com_ctre_phoenix_platform_can_PlatformCANJNI_JNI_1SetAutocacheLevel(JNIEnv *, jclass, jint);

#ifdef __cplusplus
}
#endif
#endif
