
 

// This file is autogenerated. DO NOT EDIT

#pragma once
#include <robotpy_build.h>


#include <../src/rpy/Notifier.h>





namespace rpygen {

using namespace frc;



template <typename CfgBase = EmptyTrampolineCfg>
struct PyTrampolineCfg_frc__PyNotifier :
    CfgBase
{
    using Base = frc::PyNotifier;

};


template <typename PyTrampolineBase, typename PyTrampolineCfg>
struct PyTrampoline_frc__PyNotifier : PyTrampolineBase, virtual py::trampoline_self_life_support {
    using PyTrampolineBase::PyTrampolineBase;






};

}; // namespace rpygen
