/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class com_ctre_phoenix_sensors_PigeonImuJNI */

#ifndef _Included_com_ctre_phoenix_sensors_PigeonImuJNI
#define _Included_com_ctre_phoenix_sensors_PigeonImuJNI
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_new_PigeonImu_Talon
 * Signature: (ILjava/lang/String;)J
 */
JNIEXPORT jlong JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1new_1PigeonImu_1Talon
  (JNIEnv *, jclass, jint, jstring);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_new_PigeonImu
 * Signature: (ILjava/lang/String;Ljava/lang/String;)J
 */
JNIEXPORT jlong JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1new_1PigeonImu
  (JNIEnv *, jclass, jint, jstring, jstring);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_destroy_PigeonImu
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1destroy_1PigeonImu
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_ConfigSetCustomParam
 * Signature: (JIII)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1ConfigSetCustomParam
  (JNIEnv *, jclass, jlong, jint, jint, jint);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_ConfigGetCustomParam
 * Signature: (JII)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1ConfigGetCustomParam
  (JNIEnv *, jclass, jlong, jint, jint);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_ConfigSetParameter
 * Signature: (JIDIII)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1ConfigSetParameter
  (JNIEnv *, jclass, jlong, jint, jdouble, jint, jint, jint);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_ConfigGetParameter
 * Signature: (JIII)D
 */
JNIEXPORT jdouble JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1ConfigGetParameter
  (JNIEnv *, jclass, jlong, jint, jint, jint);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_ConfigFactoryDefault
 * Signature: (JI)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1ConfigFactoryDefault
  (JNIEnv *, jclass, jlong, jint);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_SetStatusFramePeriod
 * Signature: (JIII)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1SetStatusFramePeriod
  (JNIEnv *, jclass, jlong, jint, jint, jint);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_SetYaw
 * Signature: (JDI)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1SetYaw
  (JNIEnv *, jclass, jlong, jdouble, jint);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_AddYaw
 * Signature: (JDI)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1AddYaw
  (JNIEnv *, jclass, jlong, jdouble, jint);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_SetYawToCompass
 * Signature: (JI)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1SetYawToCompass
  (JNIEnv *, jclass, jlong, jint);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_SetFusedHeading
 * Signature: (JDI)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1SetFusedHeading
  (JNIEnv *, jclass, jlong, jdouble, jint);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_AddFusedHeading
 * Signature: (JDI)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1AddFusedHeading
  (JNIEnv *, jclass, jlong, jdouble, jint);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_SetFusedHeadingToCompass
 * Signature: (JI)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1SetFusedHeadingToCompass
  (JNIEnv *, jclass, jlong, jint);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_SetAccumZAngle
 * Signature: (JDI)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1SetAccumZAngle
  (JNIEnv *, jclass, jlong, jdouble, jint);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_SetTemperatureCompensationDisable
 * Signature: (JII)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1SetTemperatureCompensationDisable
  (JNIEnv *, jclass, jlong, jint, jint);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_SetCompassDeclination
 * Signature: (JDI)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1SetCompassDeclination
  (JNIEnv *, jclass, jlong, jdouble, jint);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_SetCompassAngle
 * Signature: (JDI)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1SetCompassAngle
  (JNIEnv *, jclass, jlong, jdouble, jint);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_EnterCalibrationMode
 * Signature: (JII)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1EnterCalibrationMode
  (JNIEnv *, jclass, jlong, jint, jint);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_GetGeneralStatus
 * Signature: (J[D)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1GetGeneralStatus
  (JNIEnv *, jclass, jlong, jdoubleArray);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_Get6dQuaternion
 * Signature: (J[D)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1Get6dQuaternion
  (JNIEnv *, jclass, jlong, jdoubleArray);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_GetYawPitchRoll
 * Signature: (J[D)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1GetYawPitchRoll
  (JNIEnv *, jclass, jlong, jdoubleArray);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_GetYawPitchRoll2
 * Signature: (J[D)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1GetYawPitchRoll2
  (JNIEnv *, jclass, jlong, jdoubleArray);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_GetYaw
 * Signature: (J)D
 */
JNIEXPORT jdouble JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1GetYaw
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_GetPitch
 * Signature: (J)D
 */
JNIEXPORT jdouble JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1GetPitch
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_GetRoll
 * Signature: (J)D
 */
JNIEXPORT jdouble JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1GetRoll
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_GetGravityVector
 * Signature: (J[D)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1GetGravityVector
  (JNIEnv *, jclass, jlong, jdoubleArray);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_GetAccumGyro
 * Signature: (J[D)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1GetAccumGyro
  (JNIEnv *, jclass, jlong, jdoubleArray);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_GetAbsoluteCompassHeading
 * Signature: (J)D
 */
JNIEXPORT jdouble JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1GetAbsoluteCompassHeading
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_GetCompassHeading
 * Signature: (J)D
 */
JNIEXPORT jdouble JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1GetCompassHeading
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_GetCompassFieldStrength
 * Signature: (J)D
 */
JNIEXPORT jdouble JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1GetCompassFieldStrength
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_GetTemp
 * Signature: (J)D
 */
JNIEXPORT jdouble JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1GetTemp
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_GetUpTime
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1GetUpTime
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_GetRawMagnetometer
 * Signature: (J[S)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1GetRawMagnetometer
  (JNIEnv *, jclass, jlong, jshortArray);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_GetBiasedMagnetometer
 * Signature: (J[S)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1GetBiasedMagnetometer
  (JNIEnv *, jclass, jlong, jshortArray);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_GetBiasedAccelerometer
 * Signature: (J[S)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1GetBiasedAccelerometer
  (JNIEnv *, jclass, jlong, jshortArray);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_GetRawGyro
 * Signature: (J[D)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1GetRawGyro
  (JNIEnv *, jclass, jlong, jdoubleArray);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_GetAccelerometerAngles
 * Signature: (J[D)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1GetAccelerometerAngles
  (JNIEnv *, jclass, jlong, jdoubleArray);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_GetFusedHeading
 * Signature: (J[D)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1GetFusedHeading
  (JNIEnv *, jclass, jlong, jdoubleArray);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_GetState
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1GetState
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_GetResetCount
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1GetResetCount
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_GetResetFlags
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1GetResetFlags
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_GetFirmwareVersion
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1GetFirmwareVersion
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_GetLastError
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1GetLastError
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_GetLastTimestamp
 * Signature: (J)D
 */
JNIEXPORT jdouble JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1GetLastTimestamp
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_HasResetOccurred
 * Signature: (J)Z
 */
JNIEXPORT jboolean JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1HasResetOccurred
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_GetStatusFramePeriod
 * Signature: (JII)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1GetStatusFramePeriod
  (JNIEnv *, jclass, jlong, jint, jint);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_SetControlFramePeriod
 * Signature: (JII)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1SetControlFramePeriod
  (JNIEnv *, jclass, jlong, jint, jint);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_GetFaults
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1GetFaults
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_GetStickyFaults
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1GetStickyFaults
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_sensors_PigeonImuJNI
 * Method:    JNI_ClearStickyFaults
 * Signature: (JI)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_sensors_PigeonImuJNI_JNI_1ClearStickyFaults
  (JNIEnv *, jclass, jlong, jint);

#ifdef __cplusplus
}
#endif
#endif
