
 

// This file is autogenerated. DO NOT EDIT

#pragma once
#include <robotpy_build.h>


#include <../../build/temp.linux-armv7l-3.10/dlstatic/revlib/include/rev/SparkMaxLimitSwitch.h>





namespace rpygen {

using namespace rev;



template <typename CfgBase = EmptyTrampolineCfg>
struct PyTrampolineCfg_rev__SparkMaxLimitSwitch :
    CfgBase
{
    using Base = rev::SparkMaxLimitSwitch;

    using override_base_KGet_v = rev::SparkMaxLimitSwitch;
    using override_base_EnableLimitSwitch_b = rev::SparkMaxLimitSwitch;
    using override_base_KIsLimitSwitchEnabled_v = rev::SparkMaxLimitSwitch;
};


template <typename PyTrampolineBase, typename PyTrampolineCfg>
struct PyTrampoline_rev__SparkMaxLimitSwitch : PyTrampolineBase, virtual py::trampoline_self_life_support {
    using PyTrampolineBase::PyTrampolineBase;



#ifndef RPYGEN_DISABLE_KGet_v
    bool Get() const override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_KGet_v;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(bool), LookupBase,
            "get", );
        return CxxCallBase::Get();
    }
#endif

#ifndef RPYGEN_DISABLE_EnableLimitSwitch_b
    REVLibError EnableLimitSwitch(bool enable) override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_EnableLimitSwitch_b;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(REVLibError), LookupBase,
            "enableLimitSwitch", enable);
        return CxxCallBase::EnableLimitSwitch(enable);
    }
#endif

#ifndef RPYGEN_DISABLE_KIsLimitSwitchEnabled_v
    bool IsLimitSwitchEnabled() const override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_KIsLimitSwitchEnabled_v;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(bool), LookupBase,
            "isLimitSwitchEnabled", );
        return CxxCallBase::IsLimitSwitchEnabled();
    }
#endif




};

}; // namespace rpygen
