# This file is automatically generated, DO NOT EDIT
# fmt: off

from os.path import abspath, join, dirname, exists
_root = abspath(dirname(__file__))

# runtime dependencies
import wpiutil._init_wpiutil
import hal._init_wpiHal
import wpilib._impl._init_wpilibc
import wpilib._init_wpilib
import ctre._cci._init_ctre_cci
import ctre._api._init_ctre_api
import ctre._wpiapi._init_ctre_wpiapi
