
 

// This file is autogenerated. DO NOT EDIT

#pragma once
#include <robotpy_build.h>


#include <../../_impl/include/frc/spline/CubicHermiteSpline.h>




#include <rpygen/frc__Spline.hpp>

namespace rpygen {

using namespace frc;


template <typename CfgBase>
using PyTrampolineCfgBase_frc__CubicHermiteSpline =
    PyTrampolineCfg_frc__Spline<3, 
CfgBase
>;

template <typename CfgBase = EmptyTrampolineCfg>
struct PyTrampolineCfg_frc__CubicHermiteSpline :
    PyTrampolineCfgBase_frc__CubicHermiteSpline< CfgBase>
{
    using Base = frc::CubicHermiteSpline;

};


template <typename PyTrampolineBase, typename PyTrampolineCfg>
using PyTrampolineBase_frc__CubicHermiteSpline =
    PyTrampoline_frc__Spline<
        PyTrampolineBase
        , 3
        , PyTrampolineCfg
    >
;

template <typename PyTrampolineBase, typename PyTrampolineCfg>
struct PyTrampoline_frc__CubicHermiteSpline : PyTrampolineBase_frc__CubicHermiteSpline<PyTrampolineBase, PyTrampolineCfg> {
    using PyTrampolineBase_frc__CubicHermiteSpline<PyTrampolineBase, PyTrampolineCfg>::PyTrampolineBase_frc__CubicHermiteSpline;






};

}; // namespace rpygen
