
 

// This file is autogenerated. DO NOT EDIT

#pragma once
#include <robotpy_build.h>


#include <../_impl/include/frc/smartdashboard/MechanismRoot2d.h>

#include <frc/smartdashboard/MechanismLigament2d.h>


#define RPYGEN_DISABLE_UpdateEntries_TNetworkTable_


namespace rpygen {

using namespace frc;



template <typename CfgBase = EmptyTrampolineCfg>
struct PyTrampolineCfg_frc__MechanismRoot2d :
    CfgBase
{
    using Base = frc::MechanismRoot2d;

};


template <typename PyTrampolineBase, typename PyTrampolineCfg>
struct PyTrampoline_frc__MechanismRoot2d : PyTrampolineBase, virtual py::trampoline_self_life_support {
    using PyTrampolineBase::PyTrampolineBase;






};

}; // namespace rpygen
