
 

// This file is autogenerated. DO NOT EDIT

#pragma once
#include <robotpy_build.h>


#include <../_impl/include/frc/TimedRobot.h>




#include <rpygen/frc__IterativeRobotBase.hpp>

namespace rpygen {

using namespace frc;


template <typename CfgBase>
using PyTrampolineCfgBase_frc__TimedRobot =
    PyTrampolineCfg_frc__IterativeRobotBase<
CfgBase
>;

template <typename CfgBase = EmptyTrampolineCfg>
struct PyTrampolineCfg_frc__TimedRobot :
    PyTrampolineCfgBase_frc__TimedRobot< CfgBase>
{
    using Base = frc::TimedRobot;

    using override_base_StartCompetition_v = frc::TimedRobot;
    using override_base_EndCompetition_v = frc::TimedRobot;
};


template <typename PyTrampolineBase, typename PyTrampolineCfg>
using PyTrampolineBase_frc__TimedRobot =
    PyTrampoline_frc__IterativeRobotBase<
        PyTrampolineBase
        
        , PyTrampolineCfg
    >
;

template <typename PyTrampolineBase, typename PyTrampolineCfg>
struct PyTrampoline_frc__TimedRobot : PyTrampolineBase_frc__TimedRobot<PyTrampolineBase, PyTrampolineCfg> {
    using PyTrampolineBase_frc__TimedRobot<PyTrampolineBase, PyTrampolineCfg>::PyTrampolineBase_frc__TimedRobot;



#ifndef RPYGEN_DISABLE_StartCompetition_v
    void StartCompetition() override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_StartCompetition_v;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(void), LookupBase,
            "startCompetition", );
        return CxxCallBase::StartCompetition();
    }
#endif

#ifndef RPYGEN_DISABLE_EndCompetition_v
    void EndCompetition() override {
        using LookupBase = typename PyTrampolineCfg::Base;
        using CxxCallBase = typename PyTrampolineCfg::override_base_EndCompetition_v;
        PYBIND11_OVERRIDE_IMPL(PYBIND11_TYPE(void), LookupBase,
            "endCompetition", );
        return CxxCallBase::EndCompetition();
    }
#endif




};

}; // namespace rpygen
