
 

// This file is autogenerated. DO NOT EDIT

#pragma once
#include <robotpy_build.h>


#include <../src/include/frc2/command/CommandGroupBase.h>

#include <frc2/command/Command.h>
#include <frc2/command/Subsystem.h>



#include <rpygen/frc2__CommandBase.hpp>

namespace rpygen {

using namespace frc2;


template <typename CfgBase>
using PyTrampolineCfgBase_frc2__CommandGroupBase =
    PyTrampolineCfg_frc2__CommandBase<
CfgBase
>;

template <typename CfgBase = EmptyTrampolineCfg>
struct PyTrampolineCfg_frc2__CommandGroupBase :
    PyTrampolineCfgBase_frc2__CommandGroupBase< CfgBase>
{
    using Base = frc2::CommandGroupBase;

    using override_base_AddCommands_OTshared_ptr_Command__ = frc2::CommandGroupBase;
};


template <typename PyTrampolineBase, typename PyTrampolineCfg>
using PyTrampolineBase_frc2__CommandGroupBase =
    PyTrampoline_frc2__CommandBase<
        PyTrampolineBase
        
        , PyTrampolineCfg
    >
;

template <typename PyTrampolineBase, typename PyTrampolineCfg>
struct PyTrampoline_frc2__CommandGroupBase : PyTrampolineBase_frc2__CommandGroupBase<PyTrampolineBase, PyTrampolineCfg> {
    using PyTrampolineBase_frc2__CommandGroupBase<PyTrampolineBase, PyTrampolineCfg>::PyTrampolineBase_frc2__CommandGroupBase;



#ifndef RPYGEN_DISABLE_AddCommands_OTshared_ptr_Command__
    void AddCommands(std::vector<std::shared_ptr<Command> >&& commands) override {
        using LookupBase = typename PyTrampolineCfg::Base;
        RPYBUILD_OVERRIDE_PURE_NAME(CommandGroupBase, PYBIND11_TYPE(void), LookupBase,
            "addCommands", AddCommands, commands);
    }
#endif




};

}; // namespace rpygen
