from . import _init_ntcore

# autogenerated by 'robotpy-build create-imports ntcore ntcore._ntcore'
from ._ntcore import (
    ConnectionInfo,
    ConnectionNotification,
    EntryInfo,
    EntryNotification,
    LogMessage,
    NetworkTable,
    NetworkTableEntry,
    NetworkTablesInstance,
    NetworkTableType,
    Value,
)

__all__ = [
    "ConnectionInfo",
    "ConnectionNotification",
    "EntryInfo",
    "EntryNotification",
    "LogMessage",
    "NetworkTable",
    "NetworkTableEntry",
    "NetworkTablesInstance",
    "NetworkTableType",
    "Value",
]

NetworkTables: NetworkTablesInstance = NetworkTablesInstance.getDefault()
__all__ += ["NetworkTables"]

try:
    from .version import version as __version__
except ImportError:
    __version__ = "master"
