/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class com_ctre_phoenix_led_CANdleJNI */

#ifndef _Included_com_ctre_phoenix_led_CANdleJNI
#define _Included_com_ctre_phoenix_led_CANdleJNI
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     com_ctre_phoenix_led_CANdleJNI
 * Method:    Create
 * Signature: (ILjava/lang/String;)J
 */
JNIEXPORT jlong JNICALL Java_com_ctre_phoenix_led_CANdleJNI_Create
  (JNIEnv *, jclass, jint, jstring);

/*
 * Class:     com_ctre_phoenix_led_CANdleJNI
 * Method:    GetLastError
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_led_CANdleJNI_GetLastError
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_led_CANdleJNI
 * Method:    GetLastTimestamp
 * Signature: (J)D
 */
JNIEXPORT jdouble JNICALL Java_com_ctre_phoenix_led_CANdleJNI_GetLastTimestamp
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_led_CANdleJNI
 * Method:    BlockSet
 * Signature: (JIIIIII)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_led_CANdleJNI_BlockSet
  (JNIEnv *, jclass, jlong, jint, jint, jint, jint, jint, jint);

/*
 * Class:     com_ctre_phoenix_led_CANdleJNI
 * Method:    SetStandardAnimation
 * Signature: (JIDDIDD)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_led_CANdleJNI_SetStandardAnimation
  (JNIEnv *, jclass, jlong, jint, jdouble, jdouble, jint, jdouble, jdouble);

/*
 * Class:     com_ctre_phoenix_led_CANdleJNI
 * Method:    SetTwoSizeAnimation
 * Signature: (JIIIIIDIII)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_led_CANdleJNI_SetTwoSizeAnimation
  (JNIEnv *, jclass, jlong, jint, jint, jint, jint, jint, jdouble, jint, jint, jint);

/*
 * Class:     com_ctre_phoenix_led_CANdleJNI
 * Method:    ModulateVBatOutput
 * Signature: (JD)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_led_CANdleJNI_ModulateVBatOutput
  (JNIEnv *, jclass, jlong, jdouble);

/*
 * Class:     com_ctre_phoenix_led_CANdleJNI
 * Method:    GetBusVoltage
 * Signature: (J)D
 */
JNIEXPORT jdouble JNICALL Java_com_ctre_phoenix_led_CANdleJNI_GetBusVoltage
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_led_CANdleJNI
 * Method:    Get5VRailVoltage
 * Signature: (J)D
 */
JNIEXPORT jdouble JNICALL Java_com_ctre_phoenix_led_CANdleJNI_Get5VRailVoltage
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_led_CANdleJNI
 * Method:    GetCurrent
 * Signature: (J)D
 */
JNIEXPORT jdouble JNICALL Java_com_ctre_phoenix_led_CANdleJNI_GetCurrent
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_led_CANdleJNI
 * Method:    GetTemperature
 * Signature: (J)D
 */
JNIEXPORT jdouble JNICALL Java_com_ctre_phoenix_led_CANdleJNI_GetTemperature
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_led_CANdleJNI
 * Method:    GetVbatModulation
 * Signature: (J)D
 */
JNIEXPORT jdouble JNICALL Java_com_ctre_phoenix_led_CANdleJNI_GetVbatModulation
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_led_CANdleJNI
 * Method:    ConfigGetParameter
 * Signature: (JIII)D
 */
JNIEXPORT jdouble JNICALL Java_com_ctre_phoenix_led_CANdleJNI_ConfigGetParameter
  (JNIEnv *, jclass, jlong, jint, jint, jint);

/*
 * Class:     com_ctre_phoenix_led_CANdleJNI
 * Method:    ConfigSetParameter
 * Signature: (JIDIII)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_led_CANdleJNI_ConfigSetParameter
  (JNIEnv *, jclass, jlong, jint, jdouble, jint, jint, jint);

/*
 * Class:     com_ctre_phoenix_led_CANdleJNI
 * Method:    ConfigGetCustomParam
 * Signature: (JII)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_led_CANdleJNI_ConfigGetCustomParam
  (JNIEnv *, jclass, jlong, jint, jint);

/*
 * Class:     com_ctre_phoenix_led_CANdleJNI
 * Method:    ConfigSetCustomParam
 * Signature: (JIII)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_led_CANdleJNI_ConfigSetCustomParam
  (JNIEnv *, jclass, jlong, jint, jint, jint);

/*
 * Class:     com_ctre_phoenix_led_CANdleJNI
 * Method:    ConfigFactoryDefault
 * Signature: (JI)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_led_CANdleJNI_ConfigFactoryDefault
  (JNIEnv *, jclass, jlong, jint);

/*
 * Class:     com_ctre_phoenix_led_CANdleJNI
 * Method:    GetFaults
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_led_CANdleJNI_GetFaults
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_led_CANdleJNI
 * Method:    GetStickyFaults
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_led_CANdleJNI_GetStickyFaults
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_led_CANdleJNI
 * Method:    ClearStickyFaults
 * Signature: (JI)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_led_CANdleJNI_ClearStickyFaults
  (JNIEnv *, jclass, jlong, jint);

/*
 * Class:     com_ctre_phoenix_led_CANdleJNI
 * Method:    GetFirmwareVersion
 * Signature: (J)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_led_CANdleJNI_GetFirmwareVersion
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_led_CANdleJNI
 * Method:    HasResetOccurred
 * Signature: (J)Z
 */
JNIEXPORT jboolean JNICALL Java_com_ctre_phoenix_led_CANdleJNI_HasResetOccurred
  (JNIEnv *, jclass, jlong);

/*
 * Class:     com_ctre_phoenix_led_CANdleJNI
 * Method:    SetStatusFramePeriod
 * Signature: (JIII)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_led_CANdleJNI_SetStatusFramePeriod
  (JNIEnv *, jclass, jlong, jint, jint, jint);

/*
 * Class:     com_ctre_phoenix_led_CANdleJNI
 * Method:    GetStatusFramePeriod
 * Signature: (JII)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_led_CANdleJNI_GetStatusFramePeriod
  (JNIEnv *, jclass, jlong, jint, jint);

/*
 * Class:     com_ctre_phoenix_led_CANdleJNI
 * Method:    SetControlFramePeriod
 * Signature: (JII)I
 */
JNIEXPORT jint JNICALL Java_com_ctre_phoenix_led_CANdleJNI_SetControlFramePeriod
  (JNIEnv *, jclass, jlong, jint, jint);

#ifdef __cplusplus
}
#endif
#endif
