
 

// This file is autogenerated. DO NOT EDIT

#pragma once
#include <robotpy_build.h>


#include <../_impl/include/frc/smartdashboard/MechanismObject2d.h>

#include <frc/smartdashboard/MechanismLigament2d.h>




namespace rpygen {

using namespace frc;



template <typename CfgBase = EmptyTrampolineCfg>
struct PyTrampolineCfg_frc__MechanismObject2d :
    CfgBase
{
    using Base = frc::MechanismObject2d;

    using override_base_UpdateEntries_TNetworkTable_ = frc::MechanismObject2d;
};


template <typename PyTrampolineBase, typename PyTrampolineCfg>
struct PyTrampoline_frc__MechanismObject2d : PyTrampolineBase, virtual py::trampoline_self_life_support {
    using PyTrampolineBase::PyTrampolineBase;



#ifndef RPYGEN_DISABLE_UpdateEntries_TNetworkTable_
    void UpdateEntries(std::shared_ptr<nt::NetworkTable > table) override {
        using LookupBase = typename PyTrampolineCfg::Base;
        RPYBUILD_OVERRIDE_PURE_NAME(MechanismObject2d, PYBIND11_TYPE(void), LookupBase,
            "_updateEntries", UpdateEntries, table);
    }
#endif




};

}; // namespace rpygen
