// Copyright (c) FIRST and other WPILib contributors.
// Open Source Software; you can modify and/or share it under the terms of
// the WPILib BSD license file in the root directory of this project.

#pragma once

/*
 * The corresponding WPILibVersion.cpp file is autogenerated by the build
 * system. If it does not exist, make sure that you run a build.
 */
extern const char* GetWPILibVersion();
